@@ -1,18 +1,52 @@
-#ifndef WRAPPED_RE2_H_
-#define WRAPPED_RE2_H_
-
-#include <nan.h>
-
-#include <re2/re2.h>
+#pragma once
 
 #include <string>
+#include <nan.h>
+#include <re2/re2.h>
+
+struct StrVal
+{
+	char *data;
+	size_t size, length;
+	size_t index, byteIndex;
+	bool isBuffer, isValidIndex, isBad;
+
+	StrVal() : data(NULL), size(0), length(0), index(0), byteIndex(0), isBuffer(false), isValidIndex(false), isBad(false) {}
+
+	operator re2::StringPiece() const { return re2::StringPiece(data, size); }
+
+	void setIndex(size_t newIndex = 0);
+	void reset(const v8::Local<v8::Value> &arg, size_t size, size_t length, size_t newIndex = 0, bool buffer = false);
+
+	void clear()
+	{
+		isBad = isBuffer = isValidIndex = false;
+		size = length = index = byteIndex = 0;
+		data = nullptr;
+	}
+};
 
 class WrappedRE2 : public Nan::ObjectWrap
 {
 private:
-	WrappedRE2(const re2::StringPiece &pattern, const re2::RE2::Options &options, const std::string &src,
-			   const bool &g, const bool &i, const bool &m, const bool &s, const bool &y, const bool &d) : regexp(pattern, options),
-																										   source(src), global(g), ignoreCase(i), multiline(m), dotAll(s), sticky(y), hasIndices(d), lastIndex(0) {}
+	WrappedRE2(
+		const re2::StringPiece &pattern,
+		const re2::RE2::Options &options,
+		const std::string &src,
+		const bool &g,
+		const bool &i,
+		const bool &m,
+		const bool &s,
+		const bool &y,
+		const bool &d) : regexp(pattern, options),
+						 source(src),
+						 global(g),
+						 ignoreCase(i),
+						 multiline(m),
+						 dotAll(s),
+						 sticky(y),
+						 hasIndices(d),
+						 lastIndex(0) {}
 
 	static NAN_METHOD(New);
 	static NAN_METHOD(ToString);
@@ -45,6 +79,8 @@ private:
 	static NAN_SETTER(SetUnicodeWarningLevel);
 
 public:
+	~WrappedRE2();
+
 	static v8::Local<v8::Function> Init();
 
 	static inline bool HasInstance(v8::Local<v8::Object> object)
@@ -73,6 +109,40 @@ public:
 	bool sticky;
 	bool hasIndices;
 	size_t lastIndex;
+
+	friend struct PrepareLastString;
+
+private:
+	Nan::Persistent<v8::Value> lastString; // weak pointer
+	Nan::Persistent<v8::Object> lastCache; // weak pointer
+	StrVal lastStringValue;
+
+	void dropCache();
+	const StrVal &prepareArgument(const v8::Local<v8::Value> &arg, bool ignoreLastIndex = false);
+	void doneWithLastString();
+
+	friend class PrepareLastString;
+};
+
+struct PrepareLastString
+{
+	PrepareLastString(WrappedRE2 *re2, const v8::Local<v8::Value> &arg, bool ignoreLastIndex = false) : re2(re2) {
+		re2->prepareArgument(arg, ignoreLastIndex);
+	}
+
+	~PrepareLastString() {
+		re2->doneWithLastString();
+	}
+
+	operator const StrVal&() const {
+		return re2->lastStringValue;
+	}
+
+	operator StrVal&() {
+		return re2->lastStringValue;
+	}
+
+	WrappedRE2 *re2;
 };
 
 // utilities
@@ -153,4 +223,15 @@ inline size_t getUtf8CharSize(char ch)
 	return ((0xE5000000 >> ((ch >> 3) & 0x1E)) & 3) + 1;
 }
 
-#endif
+inline size_t getUtf16PositionByCounter(const char *data, size_t from, size_t n)
+{
+	for (; n > 0; --n)
+	{
+		size_t s = getUtf8CharSize(data[from]);
+		from += s;
+		if (s == 4 && n >= 2)
+			--n; // this utf8 character will take two utf16 characters
+				 // the decrement above is protected to avoid an overflow of an unsigned integer
+	}
+	return from;
+}
